"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _asn1js = require("asn1js");

var asn1js = _interopRequireWildcard(_asn1js);

var _pvutils = require("pvutils");

var _pkijs = require("pkijs");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

//**************************************************************************************
// noinspection JSUnusedGlobalSymbols
var CAdESCTimestamp = function (_ContentInfo) {
	_inherits(CAdESCTimestamp, _ContentInfo);

	//**********************************************************************************
	/**
  * Constructor for CAdESCTimestamp class
  * @param {Object} [parameters={}]
  * @property {Object} [schema] asn1js parsed value
  */
	function CAdESCTimestamp() {
		var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, CAdESCTimestamp);

		/**
   * @type {ArrayBuffer}
   * @description tspResponse
   */
		var _this = _possibleConstructorReturn(this, (CAdESCTimestamp.__proto__ || Object.getPrototypeOf(CAdESCTimestamp)).call(this, parameters));

		_this.tspResponse = (0, _pvutils.getParametersValue)(parameters, "tspResponse", new ArrayBuffer(0));
		return _this;
	}
	//**********************************************************************************
	// noinspection JSUnusedGlobalSymbols
	/**
  * Get "ArrayBuffer" to transfer to time-stamp server
  * @param {SignedData} cmsSignedData CMS Signed Data to make attribute for
  * @param {number} signerIndex Index of signer to make attribute for
  * @param {Object} [parameters] Additional parameters for making attribute
  * @returns {Promise}
  */


	_createClass(CAdESCTimestamp, [{
		key: "getStampingBuffer",
		value: function getStampingBuffer(cmsSignedData, signerIndex) {
			var parameters = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

			//region Initial variables
			var sequence = Promise.resolve();

			var hashAlgorithm = "SHA-256";

			var resultBuffer = new ArrayBuffer(0);

			var signatureTimeStamp = void 0; // SignatureTimeStamp
			var completeCertificateReferences = void 0; // CompleteCertificateReferences
			var completeRevocationReferences = void 0; // CompleteRevocationReferences
			//endregion

			//region Get a "crypto" extension
			var crypto = (0, _pkijs.getCrypto)();
			if (typeof crypto === "undefined") return Promise.reject("Unable to create WebCrypto object");
			//endregion

			//region Check input parameters
			if ("hashAlgorithm" in parameters) hashAlgorithm = parameters.hashAlgorithm;

			if ("signatureTimeStamp" in parameters) signatureTimeStamp = parameters.signatureTimeStamp;else return Promise.reject("Parameter \"signatureTimeStamp\" is mandatory for making \"CAdES-C-Timestamp\" attribute");

			if ("completeCertificateReferences" in parameters) completeCertificateReferences = parameters.completeCertificateReferences;else return Promise.reject("Parameter \"completeCertificateReferences\" is mandatory for making \"CAdES-C-Timestamp\" attribute");

			if ("completeRevocationReferences" in parameters) completeRevocationReferences = parameters.completeRevocationReferences;else return Promise.reject("Parameter \"completeRevocationReferences\" is mandatory for making \"CAdES-C-Timestamp\" attribute");
			//endregion

			//region Make stamping buffer
			sequence = sequence.then(function () {
				resultBuffer = (0, _pvutils.utilConcatBuf)(resultBuffer, cmsSignedData.signerInfos[signerIndex].signature.valueBlock.valueHex);
				resultBuffer = (0, _pvutils.utilConcatBuf)(resultBuffer, signatureTimeStamp.makeAttribute().toSchema().toBER(false));
				resultBuffer = (0, _pvutils.utilConcatBuf)(resultBuffer, completeCertificateReferences.makeAttribute().toSchema().toBER(false));
				resultBuffer = (0, _pvutils.utilConcatBuf)(resultBuffer, completeRevocationReferences.makeAttribute().toSchema().toBER(false));
			}, function (error) {
				return Promise.reject(error);
			});
			//endregion

			//region Make hash of signature
			sequence = sequence.then(function () {
				return crypto.digest({ name: hashAlgorithm }, resultBuffer);
			}, function (error) {
				return Promise.reject(error);
			});
			//endregion

			return sequence;
		}
		//**********************************************************************************
		/**
   * Create "CAdES-C-Timestamp" CAdES attribute
   * @param {Object} parameters Additional parameters for making attribute
   * @returns {Attribute}
   */

	}, {
		key: "makeAttribute",
		value: function makeAttribute(parameters) {
			//region Initial variables
			var tspResponse = void 0;
			//endregion

			//region Check input parameters
			if ("tspResponse" in parameters) tspResponse = parameters.tspResponse;else {
				if ("tspResponse" in this) tspResponse = this.tspResponse;else throw new Error("Parameter \"tspResponse\" is mandatory for making \"CAdES-C-Timestamp\" attribute");
			}

			this.tspResponse = tspResponse;
			//endregion

			//region Change type of "tspResponse"
			var asn1 = asn1js.fromBER(tspResponse);
			tspResponse = new _pkijs.TimeStampResp({ schema: asn1.result });
			//endregion

			//region Initialize internal variables from "tspResponse"
			if ("timeStampToken" in tspResponse) this.fromSchema(tspResponse.timeStampToken.toSchema());else throw new Error("No neccessary \"timeStampToken\" inside \"tspResponse\"");
			//endregion

			//region Create and return attribute
			return new _pkijs.Attribute({
				type: "1.2.840.113549.1.9.16.2.25",
				values: [this.toSchema()]
			});
			//endregion
		}
		//**********************************************************************************

	}]);

	return CAdESCTimestamp;
}(_pkijs.ContentInfo);
//**************************************************************************************


exports.default = CAdESCTimestamp;
//# sourceMappingURL=CAdESCTimestamp.js.map