"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _pkijs = require("pkijs");

var _CompleteRevocationRefs = require("./CompleteRevocationRefs.js");

var _CompleteRevocationRefs2 = _interopRequireDefault(_CompleteRevocationRefs);

var _CrlValidatedID = require("./CrlValidatedID.js");

var _CrlValidatedID2 = _interopRequireDefault(_CrlValidatedID);

var _OcspResponsesID = require("./OcspResponsesID.js");

var _OcspResponsesID2 = _interopRequireDefault(_OcspResponsesID);

var _CrlOcspRef = require("./CrlOcspRef.js");

var _CrlOcspRef2 = _interopRequireDefault(_CrlOcspRef);

var _CRLListID = require("./CRLListID.js");

var _CRLListID2 = _interopRequireDefault(_CRLListID);

var _OcspListID = require("./OcspListID.js");

var _OcspListID2 = _interopRequireDefault(_OcspListID);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

//**************************************************************************************
// noinspection JSUnusedGlobalSymbols
var CompleteRevocationReferences = function (_CompleteRevocationRe) {
	_inherits(CompleteRevocationReferences, _CompleteRevocationRe);

	//**********************************************************************************
	/**
  * Constructor for CompleteRevocationReferences class
  * @param {Object} [parameters={}]
  * @property {Object} [schema] asn1js parsed value
  */
	function CompleteRevocationReferences() {
		var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, CompleteRevocationReferences);

		return _possibleConstructorReturn(this, (CompleteRevocationReferences.__proto__ || Object.getPrototypeOf(CompleteRevocationReferences)).call(this, parameters));
	}
	//**********************************************************************************
	/**
  * Creates "complete-revocation-references" for given CMS Signed Data and signer index
  * @param {SignedData} cmsSigned CMS Signed Data to make attribute for
  * @param {number} signerIndex Index of signer to make attribute for
  * @param {Object} parameters Additional parameters for making attribute
  * @returns {Promise}
  */


	_createClass(CompleteRevocationReferences, [{
		key: "fillValues",
		value: function fillValues(cmsSigned, signerIndex, parameters) {
			//region Initial variables
			var _this = this;
			var sequence = Promise.resolve();

			var hashAlgorithm = "SHA-1";
			var ocspResponses = []; // Array of OCSP Responses

			var crlRefs = []; // CrlValidatedID;
			var ocspRefs = []; // OcspResponsesID
			//endregion

			//region Check input parameters
			if ("hashAlgorithm" in parameters) hashAlgorithm = parameters.hashAlgorithm;

			if ("ocspResponses" in parameters) ocspResponses = parameters.ocspResponses;
			//endregion

			//region Append references for all CRLs
			if ("crls" in cmsSigned) {
				sequence = sequence.then(function () {
					var promises = [];

					for (var i = 0; i < cmsSigned.crls.length; i++) {
						crlRefs.push(new _CrlValidatedID2.default());
						promises.push(crlRefs[crlRefs.length - 1].fillValues({
							hashAlgorithm: hashAlgorithm,
							crl: cmsSigned.crls[i]
						}));
					}

					return Promise.all(promises);
				}, function (error) {
					return Promise.reject(error);
				});
			}
			//endregion

			//region Append references for all OCSPs
			if (ocspResponses.length) {
				sequence = sequence.then(function () {
					var promises = [];

					for (var i = 0; i < ocspResponses.length; i++) {
						ocspRefs.push(new _OcspResponsesID2.default());
						promises.push(ocspRefs[ocspRefs.length - 1].fillValues({
							hashAlgorithm: hashAlgorithm,
							ocspResponse: ocspResponses[i]
						}));
					}

					return Promise.all(promises);
				}, function (error) {
					return Promise.reject(error);
				});
			}
			//endregion

			//region Push all values "in place"
			return sequence.then(function () {
				if (crlRefs.length || ocspRefs.length) {
					_this.completeRevocationRefs.push(new _CrlOcspRef2.default());

					if (crlRefs.length) {
						_this.completeRevocationRefs[_this.completeRevocationRefs.length - 1].crlids = new _CRLListID2.default({
							crls: crlRefs
						});
					}

					if (ocspRefs.length) {
						_this.completeRevocationRefs[_this.completeRevocationRefs.length - 1].ocspids = new _OcspListID2.default({
							ocspResponses: ocspRefs
						});
					}
				}
			}, function (error) {
				return Promise.reject(error);
			});
			//endregion
		}
		//**********************************************************************************
		/**
   * Create "complete-revocation-references" CAdES attribute
   * @param {Object} [parameters] Additional parameters for making attribute
   * @returns {Attribute}
   */

	}, {
		key: "makeAttribute",
		value: function makeAttribute() {
			var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			//region Create and return attribute
			return new _pkijs.Attribute({
				type: "1.2.840.113549.1.9.16.2.22",
				values: [this.toSchema()]
			});
			//endregion
		}
		//**********************************************************************************

	}]);

	return CompleteRevocationReferences;
}(_CompleteRevocationRefs2.default);
//**************************************************************************************


exports.default = CompleteRevocationReferences;
//# sourceMappingURL=CompleteRevocationReferences.js.map