"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _asn1js = require("asn1js");

var asn1js = _interopRequireWildcard(_asn1js);

var _pvutils = require("pvutils");

var _pkijs = require("pkijs");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

//**************************************************************************************
// noinspection JSUnusedGlobalSymbols
var SignatureTimeStamp = function (_ContentInfo) {
	_inherits(SignatureTimeStamp, _ContentInfo);

	//**********************************************************************************
	/**
  * Constructor for SignatureTimeStamp class
  * @param {Object} [parameters={}]
  * @property {Object} [schema] asn1js parsed value
  */
	function SignatureTimeStamp() {
		var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, SignatureTimeStamp);

		//region Internal properties of the object
		var _this = _possibleConstructorReturn(this, (SignatureTimeStamp.__proto__ || Object.getPrototypeOf(SignatureTimeStamp)).call(this, parameters));

		if ("tspResponse" in parameters)
			/**
    * @type {ArrayBuffer}
    * @description tspResponse
    */
			_this.tspResponse = (0, _pvutils.getParametersValue)(parameters, "tspResponse", SignatureTimeStamp.defaultValues("tspResponse"));

		if ("tstInfo" in parameters)
			/**
    * @type {TSTInfo}
    * @description tstInfo
    */
			_this.tstInfo = (0, _pvutils.getParametersValue)(parameters, "tstInfo", SignatureTimeStamp.defaultValues("tstInfo"));
		//endregion

		//region If input argument array contains "schema" for this object
		if ("schema" in parameters) _this.fromSchema(parameters.schema);
		//endregion
		return _this;
	}
	//**********************************************************************************
	/**
  * Convert parsed asn1js object into current class
  * @param {!Object} schema
  */


	_createClass(SignatureTimeStamp, [{
		key: "fromSchema",
		value: function fromSchema(schema) {
			_get(SignatureTimeStamp.prototype.__proto__ || Object.getPrototypeOf(SignatureTimeStamp.prototype), "fromSchema", this).call(this, schema);

			//region Check "contentType"
			if (this.contentType !== "1.2.840.113549.1.7.2") // signedData
				throw new Error("Object's schema was not verified against input data for SignatureTimeStamp");
			//endregion

			//region Get internal "CMS_SIGNED_DATA"
			var cmsSigned = new _pkijs.SignedData({ schema: this.content });
			//endregion

			//region Get internal TST_INFO
			if (cmsSigned.encapContentInfo.eContentType !== "1.2.840.113549.1.9.16.1.4") throw new Error("Incorrect format for SignatureTimeStamp");

			if ("eContent" in cmsSigned.encapContentInfo === false) throw new Error("Incorrect format for SignatureTimeStamp");

			if (cmsSigned.encapContentInfo.eContent instanceof asn1js.OctetString === false) throw new Error("Incorrect format for SignatureTimeStamp");

			var asn1 = asn1js.fromBER(cmsSigned.encapContentInfo.eContent.valueBlock.valueHex);
			this.tstInfo = new _pkijs.TSTInfo({ schema: asn1.result });
			//endregion
		}
		//**********************************************************************************
		// noinspection JSUnusedGlobalSymbols
		/**
   * Get "ArrayBuffer" to transfer to time-stamp server
   * @param {SignedData} cmsSignedData CMS Signed Data to make attribute for
   * @param {number} signerIndex Index of signer to make attribute for
   * @param {Object} parameters Additional parameters for making attribute
   * @returns {Promise}
   */

	}, {
		key: "getStampingBuffer",
		value: function getStampingBuffer(cmsSignedData, signerIndex, parameters) {
			//region Initial variables
			var hashAlgorithm = "SHA-256";
			//endregion

			//region Get a "crypto" extension
			var crypto = (0, _pkijs.getCrypto)();
			if (typeof crypto === "undefined") return Promise.reject("Unable to create WebCrypto object");
			//endregion

			//region Check input parameters
			if ("hashAlgorithm" in parameters) hashAlgorithm = parameters.hashAlgorithm;
			//endregion

			return Promise.resolve().then(function () {
				return crypto.digest({ name: hashAlgorithm }, cmsSignedData.signerInfos[signerIndex].signature.valueBlock.valueHex);
			}, function (error) {
				return Promise.reject(error);
			});
		}
		//**********************************************************************************
		/**
   * Create "signature-time-stamp" CAdES attribute
   * @param {Object} [parameters] Additional parameters for making attribute
   * @returns {Attribute}
   */

	}, {
		key: "makeAttribute",
		value: function makeAttribute() {
			var parameters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			//region Initial variables
			var tspResponse = void 0;
			//endregion

			//region Check input parameters
			if ("tspResponse" in parameters) tspResponse = parameters.tspResponse;else {
				if ("tspResponse" in this) tspResponse = this.tspResponse;else throw new Error("Parameter \"tspResponse\" is mandatory for making \"signature-time-stamp\" attribute");
			}

			this.tspResponse = tspResponse;
			//endregion

			//region Change type of "tspResponse"
			var asn1 = asn1js.fromBER(tspResponse);
			tspResponse = new _pkijs.TimeStampResp({ schema: asn1.result });
			//endregion

			//region Initialize internal variables from "tspResponse"
			if ("timeStampToken" in tspResponse) this.fromSchema(tspResponse.timeStampToken.toSchema());else throw new Error("No neccessary \"timeStampToken\" inside \"tspResponse\"");
			//endregion

			//region Create and return attribute
			return new _pkijs.Attribute({
				type: "1.2.840.113549.1.9.16.2.14",
				values: [this.toSchema()]
			});
			//endregion
		}
		//**********************************************************************************

	}]);

	return SignatureTimeStamp;
}(_pkijs.ContentInfo);
//**************************************************************************************


exports.default = SignatureTimeStamp;
//# sourceMappingURL=SignatureTimeStamp.js.map